
package samplethreadcounter.web;
import java.io.IOException;
import java.util.List;

import javax.resource.spi.work.ExecutionContext;
import javax.resource.spi.work.Work;
import javax.resource.spi.work.WorkException;
import javax.resource.spi.work.WorkListener;
import javax.resource.spi.work.WorkManager;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import samplethreadcounter.ra.ResourceAdapterImpl;
import samplethreadcounter.ra.SimpleWork;
import samplethreadcounter.ra.SimpleWorkListener;
import samplethreadcounter.util.LogWrapper;

/**
* Servlet
*
* @author
* @version
*/

public class DebugServlet extends HttpServlet {


	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	WorkManager workManager = ResourceAdapterImpl.getWorkManager();

	Work works[] = null;
    
	public void service(HttpServletRequest req, HttpServletResponse res)
		throws ServletException, IOException {

		String responseText = "";

		try {
			UtilityServlet.printRequest(req);

			if(workManager == null){
				LogWrapper.info("cannot get work manager from RA.");
				return;
			}

			ParameterServlet para = new ParameterServlet(req);
			
			String methodName = para.getMethodName();
			
			if(methodName == null || methodName.trim().length() == 0 ){
				throw new ServletException("Cannot run test. 'submitButton' must be specified in the request.");
			}
			else{
				LogWrapper.info("run method " + methodName);
			}

			if(methodName.equals("checkPoolStatus")){
				responseText = this.checkPoolStatus(para);
				LogWrapper.info(responseText.replaceAll("<BR>", "\n"));
				req.setAttribute("result",responseText);
				getServletContext().getRequestDispatcher("/result.jsp").forward(req, res);
			}
			else if(methodName.equals("releaseAllWorks")){
				this.releaseAllWorks();
				getServletContext().getRequestDispatcher("/index.html").forward(req, res);
			}
			else{
				
				int workNum = para.getWorkNum();
				if(workNum > 1000){
					LogWrapper.info("Only 1000 works are allowed to be created in this test...");
					para.setWorkNum(1000);
					workNum = 1000;
				}
				
				works = new Work[workNum];
				
				LogWrapper.info("workNum " + workNum);
				
				for(int i = 0; i < workNum ; i++){
					works[i] = new SimpleWork();
					((SimpleWork)works[i]).setCounter(para.getWorkRunningTime());
					((SimpleWork)works[i]).setName("Work_id_" + i);
					((SimpleWork)works[i]).setExceptionInRun(para.isExceptionInRun());
					
					this.doAction(works[i],para);
				}
				
				getServletContext().getRequestDispatcher("/index.html").forward(req, res);
			}
	        
			responseText = "";

		} catch (Exception e) {
			// Call for the next page
			e.printStackTrace();
			getServletContext().getRequestDispatcher("/index.html").forward(req, res);
		}
		

	}
	
	private void doAction(Work work, ParameterServlet para) throws ServletException {
		// TODO Auto-generated method stub
		
		String action = para.getMethodName();
		
		try {
			if(para.isWorkActionWithPara()){

				int startTimeout = para.getStartTimeout();
				WorkListener listener = null;
				if(!para.isWorkListenerNull()){
					listener = new SimpleWorkListener();
				}
				ExecutionContext ec = null;
				if(!para.isExecutionContextNull()){
					ec = new ExecutionContext();
				}

				LogWrapper.info("run " + action + "Work with parameters. " +
						"startTimeout =" + startTimeout + ";" +
						"listener = " + listener + ";" +
						"ec = " + ec);

				
				if(action.equals("do")){
					workManager.doWork(work, startTimeout, ec, listener);
				}
				else if(action.equals("start")){
					workManager.startWork(work, startTimeout, ec, listener);
				}
				else if(action.equals("schedule")){
					workManager.scheduleWork(work, startTimeout, ec, listener);
				}
				else{
					LogWrapper.info(action + " does not exist...");
				}
	
			}
			else{
				LogWrapper.info("run " + action + "Work without parameters");

				if(action.equals("do")){
					workManager.doWork(work);
				}
				else if(action.equals("start")){
					workManager.startWork(work);
				}
				else if(action.equals("schedule")){
					workManager.scheduleWork(work);
				}
				else{
					LogWrapper.info(action + " does not exist...");
				}
				
			}
		
		} catch (WorkException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			throw new ServletException(e.toString());
		}
	}

	public void releaseAllWorks(){
		LogWrapper.info("Release all works...");
		if(works == null){
			LogWrapper.info("No work has been initialised...");
			return;
		}
		for(int i = 0; i < works.length; i++){
			if(works[i] != null){
				works[i].release();
			}
		}	
		works = null;
	}


	public String checkPoolStatus(ParameterServlet para){
		LogWrapper.info("check pool and timer status...");
		return "";
	}
}
